% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bvhar-horseshoe.R, R/print-bvharsp.R
\name{bvhar_horseshoe}
\alias{bvhar_horseshoe}
\alias{print.bvharhs}
\alias{knit_print.bvharhs}
\title{Fitting Bayesian VHAR of Horseshoe Prior}
\usage{
bvhar_horseshoe(
  y,
  har = c(5, 22),
  num_chains = 1,
  num_iter = 1000,
  num_burn = floor(num_iter/2),
  thinning = 1,
  bayes_spec = set_horseshoe(),
  include_mean = TRUE,
  minnesota = c("no", "short", "longrun"),
  algo = c("block", "gibbs"),
  verbose = FALSE,
  num_thread = 1
)

\method{print}{bvharhs}(x, digits = max(3L, getOption("digits") - 3L), ...)

\method{knit_print}{bvharhs}(x, ...)
}
\arguments{
\item{y}{Time series data of which columns indicate the variables}

\item{har}{Numeric vector for weekly and monthly order. By default, \code{c(5, 22)}.}

\item{num_chains}{Number of MCMC chains}

\item{num_iter}{MCMC iteration number}

\item{num_burn}{Number of burn-in (warm-up). Half of the iteration is the default choice.}

\item{thinning}{Thinning every thinning-th iteration}

\item{bayes_spec}{Horseshoe initialization specification by \code{\link[=set_horseshoe]{set_horseshoe()}}.}

\item{include_mean}{Add constant term (Default: \code{TRUE}) or not (\code{FALSE})}

\item{minnesota}{Minnesota type}

\item{algo}{Ordinary gibbs sampling (\code{gibbs}) or blocked gibbs (Default: \code{block}).}

\item{verbose}{Print the progress bar in the console. By default, \code{FALSE}.}

\item{num_thread}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}} Number of threads}

\item{x}{\code{bvharhs} object}

\item{digits}{digit option to print}

\item{...}{not used}
}
\value{
\code{bvhar_horseshoe} returns an object named \code{bvarhs} \link{class}.
It is a list with the following components:

\describe{
\item{coefficients}{Posterior mean of VHAR coefficients.}
\item{covmat}{Posterior mean of covariance matrix}
\item{psi_posterior}{Posterior mean of precision matrix \eqn{\Psi}}
\item{param}{\link[posterior:draws_df]{posterior::draws_df} with every variable: alpha, lambda, tau, omega, and eta}
\item{param_names}{Name of every parameter.}
\item{df}{Numer of Coefficients: \verb{3m + 1} or \verb{3m}}
\item{p}{3 (The number of terms. It contains this element for usage in other functions.)}
\item{week}{Order for weekly term}
\item{month}{Order for monthly term}
\item{m}{Dimension of the data}
\item{obs}{Sample size used when training = \code{totobs} - \code{p}}
\item{totobs}{Total number of the observation}
\item{call}{Matched call}
\item{process}{Description of the model, e.g. \code{VHAR_Horseshoe}}
\item{type}{include constant term (\code{const}) or not (\code{none})}
\item{algo}{Usual Gibbs sampling (\code{gibbs}) or fast sampling (\code{fast})}
\item{spec}{Horseshoe specification defined by \code{\link[=set_horseshoe]{set_horseshoe()}}}
\item{chain}{The numer of chains}
\item{iter}{Total iterations}
\item{burn}{Burn-in}
\item{thin}{Thinning}
\item{group}{Indicators for group.}
\item{num_group}{Number of groups.}
\item{HARtrans}{VHAR linear transformation matrix}
\item{y0}{\eqn{Y_0}}
\item{design}{\eqn{X_0}}
\item{y}{Raw input}
}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} This function fits VHAR with horseshoe prior.
}
\references{
Kim, Y. G., and Baek, C. (n.d.). Working paper.
}
